import os
import shutil
import tarfile

import requests

from svelte_web_components.paths import data_path, get_path, paths_init, lib_path


def check_setup():
    if os.path.exists(get_path("svelte_app")):
        return True
    return False


def setup_node(force=False):
    if not force and check_setup():
        print("Node already setup")
        return None

    if os.path.exists(data_path()):
        shutil.rmtree(data_path())

    paths_init()
    print("setup_node", data_path())
    shutil.copytree(lib_path("svelte_app"), get_path("svelte_app"))

    current_path = os.getcwd()
    os.chdir(data_path())

    # copy the svelte app to the app data path

    if os.path.exists("./node"):
        shutil.rmtree("./node")
    url = "https://nodejs.org/dist/v20.6.0/node-v20.6.0-linux-x64.tar.xz"
    with requests.get(url, stream=True) as rx, tarfile.open(fileobj=rx.raw, mode="r|*") as tarobj:
        tarobj.extractall()

    # rename the extracted folder to 'node'
    os.rename("./node-v20.6.0-linux-x64", "./node")
    os.chdir("./svelte_app")
    from svelte_web_components.bundle import npm_install

    npm_install()

    os.chdir(current_path)



import glob
import json
import os
import shutil
import subprocess
import time
from re import sub
import filecmp

from jinja2 import FileSystemLoader, Environment

from svelte_web_components.setup_node import setup_node


def kebab(s):
    return '-'.join(
        sub(r"(\s|_|-)+", " ",
            sub(r"[A-Z]{2,}(?=[A-Z][a-z]+[0-9]*|\b)|[A-Z]?[a-z]+[0-9]*|[A-Z]|[0-9]+",
                lambda mo: ' ' + mo.group(0).lower(), s)).split())


from svelte_web_components.paths import npm_path, node_path, get_path


def get_npm_ls():
    res = subprocess.check_output([npm_path(), "ls", "--json"])
    return json.loads(res)


def npm_install():
    if os.path.exists("./node_modules"):
        return None
    return subprocess.check_output([npm_path(), "install"])


def npm_install_extra(extra: list) -> str | None:
    if extra is None:
        return None
    packages = get_npm_ls()["dependencies"]
    for package in extra:
        if package not in packages:
            continue
        extra.remove(package)
    if len(extra) == 0:
        return None

    return subprocess.check_output([npm_path(), "install", *extra]).decode("utf-8")


def build_components_js():
    build_js = "build.mjs"
    res = subprocess.check_output([node_path(), build_js]).decode("utf-8")
    return res


def generate_import_statements2(component_path, component_js):
    import_statements = []
    export_statements = []
    for svelte_file in glob.glob(os.path.join(component_path, '**/*.svelte'), recursive=True):
        relative_path = os.path.relpath(svelte_file, component_path)
        component_name = os.path.splitext(os.path.basename(svelte_file))[0]
        import_statement = f"import {component_name} from './components/{relative_path}';"
        import_statements.append(import_statement)

        export_statements.append(component_name)

    import_statements = "\n".join(import_statements)
    export_statement = "\nexport default [{" \
                       f" {', '.join(export_statements)} " \
                       "}]"

    with open(component_js, 'w') as js_file:
        js_file.write(import_statements + export_statement)


def generate_import_statements(component_path, component_js):
    packages = []

    for svelte_file in glob.glob(os.path.join(component_path, '**/*.svelte'), recursive=True):
        relative_path = os.path.relpath(svelte_file, component_path)
        component_name = os.path.splitext(os.path.basename(svelte_file))[0]

        package = {
            "name": component_name,
            "path": relative_path,
            "tag": kebab(component_name)
        }
        packages.append(package)
    print(component_js)
    environment = Environment(loader=FileSystemLoader(get_path("svelte_app")))
    template = environment.get_template("main.js.jinja2")
    rendered = template.render(packages=packages)
    print(rendered)

    with open(component_js, 'w+') as js_file:
        js_file.write(rendered)


def copy_components_path(components_path: str | os.PathLike):
    if os.path.exists(get_path("svelte_app/components")):
        shutil.rmtree(get_path("svelte_app/components"))
    shutil.copytree(components_path, get_path("svelte_app/components"))


def get_components_js(components_path: str | os.PathLike, extra_packages: list | None = None) -> str:
    components_path = os.path.abspath(components_path)
    current_path = os.getcwd()
    os.chdir(get_path("./svelte_app"))
    copy_components_path(components_path)
    # set env variables

    generate_import_statements(get_path("svelte_app/components"),
                               get_path("svelte_app/components.js"))

    npm_install_extra(extra_packages)
    res = build_components_js()
    os.chdir(current_path)
    return res


class Bundle:
    def __init__(self, components: dict[str, str | os.PathLike], extra_packages: list | None = None):
        self.components = components
        self.extra_packages = extra_packages
        self.bundled = {}
        setup_node()
        self.build()

    def build(self):
        for component_name, component_path in self.components.items():
            self.bundled[component_name] = get_components_js(component_path, self.extra_packages)

    def __getitem__(self, item):
        return self.bundled[item]


if __name__ == "__main__":
    # # overwrite the contents of the svelte_app folder
    # shutil.copytree("./svelte_app", get_path("svelte_app"), dirs_exist_ok=True)
    #
    # generate_import_statements("/home/panos/WebstormProjects/svelte_test/src/components",
    #                            "/home/panos/WebstormProjects/svelte_test/src/components.js")
    #
    # b = Bundle({"comp": "/home/panos/Downloads/comp"}, ["moment"])
    # print(b["comp"])
    shutil.copytree("/home/panos/Downloads/comp", get_path("svelte_app/components"), dirs_exist_ok=True)
    for i in range(100):
        s_time = time.time()
        print(filecmp.dircmp("/home/panos/Downloads/comp", get_path("svelte_app/components")).diff_files)
        print((time.time() - s_time) * 1000, "ms")



import os
import typing
from datetime import datetime
from email.utils import formatdate
from urllib.parse import quote

from fastapi import FastAPI
from starlette._compat import md5_hexdigest
from starlette.datastructures import Headers
from starlette.responses import Response
from starlette.types import Receive, Scope, Send

from svelte_web_components.bundle import Bundle

PathLike = typing.Union[str, "os.PathLike[str]"]


class NotModifiedResponse(Response):
    NOT_MODIFIED_HEADERS = (
        "cache-control",
        "content-location",
        "date",
        "etag",
        "expires",
        "vary",
    )

    def __init__(self, headers: Headers):
        super().__init__(
            status_code=304,
            headers={
                name: value
                for name, value in headers.items()
                if name in self.NOT_MODIFIED_HEADERS
            },
        )


def is_not_modified(
        response_headers: Headers, request_headers: Headers
) -> bool:
    """
    Given the request and response headers, return `True` if an HTTP
    "Not Modified" response could be returned instead.
    """
    try:
        if_none_match = request_headers["if-none-match"]
        etag = response_headers["etag"]
        if if_none_match == etag:
            print("if_none_match == etag")
            return True
    except KeyError:
        pass

    return False


class SvelteFiles:
    def __init__(self, components: dict[str, str | os.PathLike], extra_packages: list | None = None,
                 app: FastAPI = None,
                 templates=None):
        self.bundler = Bundle(components)
        self.extra_packages = extra_packages
        self.last_modified = datetime.now()

        if templates is not None:
            self.inject_into_templates(templates)

        if app is not None:
            self.mount(app)

    def inject_into_templates(self, templates):
        template_script = '<script src="{component_name}/" type="module"></script>'

        templates.env.globals["svelte_script"] = {
            component_name: template_script.format(component_name=component_name)
            for component_name in self.bundler.components.keys()
        }
        templates.env.globals["svelte_scripts"] = "\n".join(
            template_script.format(component_name=component_name)
            for component_name in self.bundler.components.keys()
        )

    def mount(self, app):
        for component_name in self.bundler.components.keys():
            app.mount(component_name, self)

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """
        The ASGI entry point.
        """
        assert scope["type"] == "http"
        component_name = scope["raw_path"].decode("utf-8").rstrip("/")

        response = Response(self.bundler[component_name], media_type="application/javascript")

        stat = self.get_stat(component_name)
        content_disposition = f'inline; filename="{quote(component_name) + ".js"}"'

        response.headers.setdefault("content-disposition", content_disposition)

        response.headers.setdefault("content-length", stat["content_length"])
        response.headers.setdefault("last-modified", stat["last_modified"])
        response.headers.setdefault("etag", stat["etag"])

        if is_not_modified(response.headers, request_headers=Headers(scope=scope)):
            print("NotModifiedResponse")
            response = NotModifiedResponse(response.headers)

        await response(scope, receive, send)

    def get_stat(self, component_name: str) -> dict[str, str]:
        content_length = str(len(self.bundler[component_name]))
        last_modified = formatdate(self.last_modified.timestamp(), usegmt=True)
        etag_base = f"{last_modified}{content_length}"
        etag = md5_hexdigest(etag_base.encode(), usedforsecurity=False)
        return {"content_length": content_length, "last_modified": last_modified, "etag": etag}



import click

from svelte_web_components.setup_node import setup_node


@click.command()
@click.option('--clean', is_flag=True, help='Remove current setup and re-install')
def init(clean):
    setup_node(clean)


if __name__ == '__main__':
    init()



#!/usr/bin/env python

import pathlib

from setuptools import setup, find_packages

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.readlines()

readme = pathlib.Path('README.md').read_text()

setup(
    author="Panos Stavrianos",
    author_email='panos@orbitsystems.gr',
    python_requires='>=3.6',
    description="Svelte web components for python",
    long_description=readme,
    long_description_content_type='text/markdown',
    install_requires=requirements,
    setup_requires=[
        'requests', 'Jinja2'
    ],
    license="MIT license",
    include_package_data=True,
    keywords=['svelte', 'web components', 'python', 'fastapi', 'flask'],
    name='svelte_web_components',
    packages=find_packages(include=['svelte_web_components', 'svelte_web_components.*']),
    url='https://github.com/panos-stavrianos/svelte_web_components',
    version='0.0.21',
    zip_safe=False,
    extras_require={
        'fastapi': ['fastapi', 'uvicorn', 'jinja2', 'starlette'],
    },
    entry_points='''
   [console_scripts]
   svelte_wc=svelte_web_components.cli:init
   ''',
)



import os
import pathlib
import shutil
from pathlib import Path

project_name = ".svelte_web_components"


def paths_init():
    if os.path.exists(data_path()):
        shutil.rmtree(data_path())
    os.mkdir(data_path())


def data_path():
    return os.path.join(str(Path.home()), project_name)


def lib_path(path):
    p = pathlib.Path(__file__).parent.resolve()
    return os.path.join(p, path)


def get_path(path):
    return os.path.join(data_path(), path)


def npm_path():
    return os.path.join(data_path(), "node/bin/npm")


def node_path():
    return os.path.join(data_path(), "node/bin/node")
